@@ -3,7 +3,10 @@ import datetime
 
 # local imports
 from .state import State
-from .error import StateItemExists
+from .error import (
+        StateItemExists,
+        StateLockedKey,
+        )
 
 
 class PersistedState(State):
@@ -34,13 +37,14 @@ class PersistedState(State):
 
         See shep.state.State.put
         """
-        to_state = super(PersistedState, self).put(key, state=state, contents=contents)
-
-        k = self.name(to_state)
+        k = self.to_name(state)
 
         self.__ensure_store(k)
+
         self.__stores[k].put(key, contents)
 
+        super(PersistedState, self).put(key, state=state, contents=contents)
+
         self.register_modify(key)
 
 
@@ -56,10 +60,15 @@ class PersistedState(State):
         k_to = self.name(to_state)
         self.__ensure_store(k_to)
 
-        contents = self.__stores[k_from].get(key)
-        self.__stores[k_to].put(key, contents)
-        self.__stores[k_from].remove(key)
-
+        contents = None
+        try:
+            contents = self.__stores[k_from].get(key)
+            self.__stores[k_to].put(key, contents)
+            self.__stores[k_from].remove(key)
+        except StateLockedKey as e:
+            super(PersistedState, self).unset(key, or_state, allow_base=True)
+            raise e
+       
         self.sync(to_state)
 
         return to_state
@@ -135,7 +144,7 @@ class PersistedState(State):
         return to_state
 
 
-    def sync(self, state=None):
+    def sync(self, state=None, not_state=None):
         """Reload resources for a single state in memory from the persisted state store.
 
         :param state: State to load
@@ -143,11 +152,20 @@ class PersistedState(State):
         :raises StateItemExists: A content key is already recorded with a different state in memory than in persisted store.
         # :todo: if sync state is none, sync all
         """
-        states = []
+
+        states_numeric = []
         if state == None:
-            states = list(self.all())
+            states_numeric = list(self.all(numeric=True))
         else:
-            states = [self.name(state)]
+            states_numeric = [state]
+       
+        states = []
+        for state in states_numeric:
+            if not_state != None:
+                if state & not_state == 0:
+                    states.append(self.name(state))
+            else:
+                states.append(self.name(state))
 
         ks = []
         for k in states:
@@ -208,10 +226,11 @@ class PersistedState(State):
 
         See shep.state.State.replace
         """
-        super(PersistedState, self).replace(key, contents)
         state = self.state(key)
         k = self.name(state)
-        return self.__stores[k].replace(key, contents)
+        r = self.__stores[k].replace(key, contents)
+        super(PersistedState, self).replace(key, contents)
+        return r
 
 
     def modified(self, key):
