@@ -185,6 +185,12 @@ class State:
         self.__set(k, v)
 
 
+    def to_name(self, k):
+        if k == None:
+            k = 0
+        return self.name(k)
+
+
     def __alias(self, k, *args):
         v = 0
         for a in args:
@@ -211,14 +217,15 @@ class State:
         return self.__alias(k, *args)    
 
 
-    def all(self, pure=False):
-        """Return list of all unique atomic and alias states.
+    def all(self, pure=False, numeric=False):
+        """Return list of all unique atomic and alias state strings.
         
         :rtype: list of ints
         :return: states
         """
         l = []
         for k in dir(self):
+            state = None
             if k[0] == '_':
                 continue
             if k.upper() != k:
@@ -227,7 +234,12 @@ class State:
                 state = self.from_name(k)
                 if not self.__is_pure(state):
                     continue
-            l.append(k)
+            if numeric:
+                if state == None:
+                    state = self.from_name(k)
+                l.append(state)
+            else:
+                l.append(k)
         l.sort()
         return l
 
@@ -436,7 +448,7 @@ class State:
         return self.__move(key, current_state, to_state)
 
 
-    def unset(self, key, not_state):
+    def unset(self, key, not_state, allow_base=False):
         """Unset a single bit, moving to a pure or alias state.
         
         The resulting state cannot be State.base_state_name (0).
@@ -462,7 +474,7 @@ class State:
         if to_state == current_state:
             raise ValueError('invalid change for state {}: {}'.format(key, not_state))
 
-        if to_state == getattr(self, self.base_state_name):
+        if to_state == getattr(self, self.base_state_name) and not allow_base:
             raise ValueError('State {} for {} cannot be reverted to {}'.format(current_state, key, self.base_state_name))
 
         new_state = self.__reverse.get(to_state)
@@ -622,3 +634,23 @@ class State:
         statemask = ~statemask
         statemask &= self.__limit
         return statemask
+
+
+    def purge(self, key):
+        state = self.state(key)
+        state_name = self.name(state)
+
+        v = self.__keys.get(state)
+        v.remove(key) 
+
+        del self.__keys_reverse[key]
+
+        try:
+            del self.__contents[key]
+        except KeyError:
+            pass
+
+        try:
+            del self.modified_last[key]
+        except KeyError:
+            pass
